% Multivariate Receptive Field 
% Evaluating STRF using: spectrogram & sound envelope
% For sake of simplicity, in this excample is no crros-validation 
% Artur.Luczak@yale.edu    Oct.2003

clear all

% load data: spectrogram (spectrogr), neuron's response (psth), sound envelope (ampl)
load STRFdata  

i = 11:size(spectrogr,2);

% converting window of spectrogram to row vector
spec = [spectrogr(:, i)'   spectrogr(:, i-1)' spectrogr(:, i-2)'  spectrogr(:, i-3)' ...
        spectrogr(:, i-4)' spectrogr(:, i-5)' spectrogr(:, i-6)' spectrogr(:, i-7)' ...
        spectrogr(:, i-8)' spectrogr(:, i-9)' spectrogr(:, i-10)'];  
psth = psth(i);             % neuron's response 
ampl = ampl( i );           % amplitude of sound envelope

[pc,scores,latent,tsquare] = princomp(spec);    % PCA

lv = 16;        % number of principal components (85.25% variance)

a = [ scores(:,1:lv)  ampl' ] \ psth';  % Least Squares 

strf_pca = pc(:,1:lv)*a(1:lv);    %recovering STRF form LS on PCA  

sz = size(spectrogr,1);
for i = 1:size(spec,2)/sz    % converting STRF from row vector to Freq./Time form
    strf(:,i) = strf_pca(sz*(i-1)+1:sz*i);
end

figure;imagesc( strf ); axis xy % plot STRF
title('STRF')
set(gca,'xtick',[1 6 11]);  set(gca,'XTickLabel',[0 50 100]);xlabel('Time (ms)')
set(gca,'ytick',[1 20 40]); set(gca,'yTickLabel',[1 10 20]);ylabel('Freq. (kHz)')

pred =  [ scores(:,1:lv)  ampl' ] *a; % predicted neuron's response

figure;plot([psth; pred']')     % plot neuron's response and predictions
title('Neuron''s response: actual (blue) and predicted (green)')
set(gca,'xtick',[0 100 200 300 400]);  set(gca,'XTickLabel',[0 1 2 3 4]);xlabel('Time (s)')
ylabel('Firing rate (a.u)')

c = corrcoef(psth, pred);  % corr. coef. 
['correlation coefiecient: ' num2str( c(1,2) )]